/**
 * EasyBeans
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.console.registry.service;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerFactory;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

/**
 * Manages the POJO service used by flex clients.
 * @author Florent Benoit
 */
public class RegistryService implements Serializable {

    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = 7508126227963187999L;


    public RegistryService() {
    }


    protected ObjectName getObjectName(final String oNameStr) {
        ObjectName oName = null;
        try {
            oName = new ObjectName(oNameStr);
        } catch (MalformedObjectNameException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        } catch (NullPointerException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        return oName;
    }

    public List<RegistryProtocol> getRegistryProtocols() {

        // ObjectName
        String pattern = ":j2eeType=JNDIResource,*";
        ObjectName oName = getObjectName(pattern);

        Set<ObjectName> objectNames = null;
        try {
            objectNames = getMBeanServerConnection().queryNames(oName, null);
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        List<RegistryProtocol> list = new ArrayList<RegistryProtocol>();

        // For each ObjectName, get data
        if (objectNames != null) {
            for (ObjectName objectName : objectNames) {
                RegistryProtocol registryProtocol = new RegistryProtocol();
                list.add(registryProtocol);

                try {
                    // Provider URL
                    String providerURL = (String) getMBeanServerConnection().getAttribute(objectName, "ProviderURL");
                    registryProtocol.setProviderURL(providerURL);

                    // JNDI Names
                    List<String> jndiNames = (List<String>) getMBeanServerConnection().getAttribute(objectName, "Names");
                    registryProtocol.setJndiNames(jndiNames);

                    // Name
                    String name = (String) getMBeanServerConnection().getAttribute(objectName, "Name");
                    registryProtocol.setName(name);
                } catch (AttributeNotFoundException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (InstanceNotFoundException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (MBeanException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (ReflectionException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }

            }
        }

        return list;
    }


    private MBeanServerConnection getMBeanServerConnection() throws Exception {
        MBeanServer mBeanServer = MBeanServer.class.cast(MBeanServerFactory.findMBeanServer(null).get(0));
        return mBeanServer;
    }

}
